# frozen_string_literal: true

require 'botz/version'
require 'active_support/all'
require 'active_model'
require 'mechanize'
require 'csv'
require 'open-uri'

#
# web bot dsl engine
#
module Botz
  extend ActiveSupport::Autoload
  autoload :Shell
  autoload :Console
  autoload :Definition
  autoload :DefinitionFile
  autoload :Binder
  autoload :Spider
  autoload :Looper
  autoload :Connector
  autoload :Result

  const_set(:Crawler, Module.new) unless const_defined?(:Crawler)

  def self.console
    require 'pry'
    Pry.start(Botz::Console.new)
  end

  def self.open(filepath)
    ::Botz::DefinitionFile.open(filepath)
  end

  def self.define(name, domain: nil, &block)
    crawler_definition = Class.new(::Botz::Definition, &block)
    crawler_definition.domain = domain
    crawler_class_name = name.to_s.camelize

    Crawler.class_eval { remove_const(crawler_class_name) } if Crawler.const_defined?(crawler_class_name)
    Crawler.const_set(crawler_class_name, crawler_definition)
    crawler_definition
  end
end
